package net.pixtaded.crab.server;
import net.pixtaded.crab.common.Crab;
import net.pixtaded.crab.common.Logs;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.SQLException;
import java.util.Scanner;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class CrabServer implements Crab {

    private ServerSocket serverSocket;
    private boolean isStopped = false;
    private boolean isProxied = false;
    private int port;
    private final Database db;
    private Logs cache = new Logs(0, "");
    public static final byte RAC_VERSION = 0x02;
    public final ExecutorService threadPool = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());

    public CrabServer() {
        this.db = new Database("data.db");
    }

    public CrabServer(int port, boolean isProxied) {
        this.db = new Database("data.db");
        this.port = port;
        this.isProxied = isProxied;
    }

    @Override
    public void run() {
        try {
            if (this.port == 0)
                setup();
            this.cache = getDb().getLogs();
            listen();
        } catch (IOException e) {
            if (!isStopped) System.err.println("Error starting server: " + e.getMessage());
        } finally {
            stop();
        }
    }

    private void setup() {
        Scanner scanner = new Scanner(System.in);
        System.out.print("Enter server port: ");
        while (true) {
            try {
                port = Integer.parseInt(scanner.nextLine());
                if (port > 0 && port <= 65535) {
                    break;
                } else {
                    System.out.println("Enter a correct port (1-65535): ");
                }
            } catch (NumberFormatException e) {
                System.out.println("Enter a correct port number: ");
            }
        }
        System.out.print("Enable PROXY protocol? (on/off): ");
        while (true) {
            String s = scanner.nextLine();
            if (s.equals("on")) {
                this.isProxied = true;
                break;
            }
            if (s.equals("off")) {
                this.isProxied = false;
                break;
            }
            System.out.println("Enter either \"on\" or \"off\".");
        }
    }

    private void listen() throws IOException {
        Scanner scanner = new Scanner(System.in);
        if (this.isProxied) {
            serverSocket = new ServerSocket(port, 0, InetAddress.getLoopbackAddress());
        } else {
            serverSocket = new ServerSocket(port);
        }
        System.out.printf("Server successfully started! Listening on port %s.\nTo stop the server, type 'q'.\n", port);
        threadPool.submit(new ServerCLI(scanner, this));
        while (!isStopped) {
            Socket socket = serverSocket.accept();
            threadPool.submit(new ServerThread(socket, this));
        }
    }

    public synchronized void stop() {
        isStopped = true;
        try {
            if (serverSocket != null) serverSocket.close();
            getDb().close();
        } catch (IOException e) {
            System.err.println("An error occured while closing the socket: " + e.getMessage());
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            System.exit(0);
        }
    }

    public synchronized Logs getCache() {
        return cache;
    }

    public synchronized void setCache(Logs cache) {
        this.cache = cache;
    }

    public Database getDb() {
        return db;
    }

    public boolean isProxied() {
        return this.isProxied;
    }
}
